searchData={"items":[{"type":"module","title":"Lace","doc":"Lace your Elixir with a finely curated blend of functional utilities,\ncrafted to complement and enhance the standard library.","ref":"Lace.html"},{"type":"module","title":"Lace.Map","doc":"Extra sweets for `Map`s","ref":"Lace.Map.html"},{"type":"function","title":"Lace.Map.move/3","doc":"Moves the entry under `old_key` to `new_key`.\nIf there was a prexisting value under `new_key`, it gets overwritten.\nIf `old_key` is missing from the map, the map is returned unchanged.","ref":"Lace.Map.html#move/3"},{"type":"function","title":"Examples - Lace.Map.move/3","doc":"iex> Lace.Map.move(%{name: \"Marie Curie\", field: \"Physics\"}, :name, :full_name)\n    %{full_name: \"Marie Curie\", field: \"Physics\"}\n\n    iex> Lace.Map.move(%{1 => \"desert\", 2 => \"lake\"}, 2, 1)\n    %{1 => \"lake\"}\n\n    iex> Lace.Map.move(%{}, \"status\", \"state\")\n    %{}","ref":"Lace.Map.html#move/3-examples"},{"type":"function","title":"Lace.Map.move_new/3","doc":"Moves the value under `old_key` to `new_key`, unless `new_key` already exists in the map.\nReturns the map unchanged if `old_key` is missing.","ref":"Lace.Map.html#move_new/3"},{"type":"function","title":"Examples - Lace.Map.move_new/3","doc":"iex> Lace.Map.move_new(%{secondary: 20}, :secondary, :primary)\n    %{primary: 20}\n\n    iex> Lace.Map.move_new(%{primary: 10, secondary: 20}, :secondary, :primary)\n    %{primary: 10, secondary: 20}","ref":"Lace.Map.html#move_new/3-examples"},{"type":"function","title":"Lace.Map.put_if/3","doc":"Puts the given `value` in `map` under `key` if `value` is non-`nil`.\n\nUseful to conditionally add non-empty fields to maps,\nin a pipeline-friendly way and without having to do a second pass\nto reject fields with empty values.\n\nIf you need custom logic to decide whether to add the value,\nuse `put_if/4`, which takes a predicate function.","ref":"Lace.Map.html#put_if/3"},{"type":"function","title":"Example - Lace.Map.put_if/3","doc":"iex> fetch_user = fn _user_id -> nil end\n    ...> fetch_project = fn project_id -> \"project-\" <> to_string(project_id) end\n    ...>\n    ...> %{}\n    ...> |> Lace.Map.put_if(:user, fetch_user.(23))\n    ...> |> Lace.Map.put_if(:project, fetch_project.(13))\n    %{project: \"project-13\"}","ref":"Lace.Map.html#put_if/3-example"},{"type":"function","title":"Lace.Map.put_if/4","doc":"Conditionally adds a value to the map based on a predicate function.\n\nThis function is similar to `put_if`, but allows for more complex\nlogic by using a predicate function that takes two parameters: the\ncurrent map and the value to be added. The predicate should return\na boolean indicating whether the value should be inserted into the map.","ref":"Lace.Map.html#put_if/4"},{"type":"function","title":"Examples - Lace.Map.put_if/4","doc":"iex> Lace.Map.put_if(%{a: 1}, :b, 2 + 3, fn _map, n -> n > 4 end)\n    %{a: 1, b: 5}\n\n    iex> Lace.Map.put_if(%{a: 1}, :b, 2, fn _, _ -> :rand.uniform(3) < 5 end)\n    %{a: 1, b: 2}\n\n    iex> Lace.Map.put_if(%{a: 1}, :b, 2, fn _, _ -> :rand.uniform(3) > 5 end)\n    %{a: 1}","ref":"Lace.Map.html#put_if/4-examples"},{"type":"module","title":"Lace.Result","doc":"A module for working with result tuples in a clear, composable, and pipeline-friendly manner.","ref":"Lace.Result.html"},{"type":"module","title":"Result Representation - Lace.Result","doc":"This module defines `t:result/0` as strictly `{:ok, any()} | {:error, any()}`,\nand the majority of functions within this module operate exclusively on these values.\n\nSeveral standard library functions and various packages in the Elixir ecosystem\nreturn values such as `:ok`, `{:ok, val, extra_info}`, `:error`, and `{:error, code, extra_info}`,\nalong with other result-like but inconsistently structured values.\nWe refer to these values as _okish_ (representing successful computations)\nand _errorish_ (indicating failures), collectively termed _resultish_.\n\nYou can use `from/1` to convert this diverse array of values into consistent `t:result/0` types,\nenabling you to leverage the utilities provided by this module.","ref":"Lace.Result.html#module-result-representation"},{"type":"module","title":"Best Practices for Error Values - Lace.Result","doc":"For error handling, we advocate for the best practice of using `{:error,  }`,\nparticularly in libraries and packages intended for use by other code.\n\nOne of the first public mentions of this approach can be found in a post on Michał Muskała's blog,\nwith a  from Andrea Leopardi (@whatyouhide, now a member of the Elixir Core Team)\nhttps://web.archive.org/web/20180414015950/http://michal.muskala.eu/2017/02/10/error-handling-in-elixir-libraries.html#comments-whatyouhide-errors\n\nThis pattern is adopted by several high-profile and well-architected libraries,\nincluding Redix, Mint, Req, among others.\nIt offers several advantages including: a consistent shape for pattern matching,\nbeing able to trivially raise exceptions or extract messages from error tuples,\nand the ability to include additional fields with details in the error value.","ref":"Lace.Result.html#module-best-practices-for-error-values"},{"type":"function","title":"Lace.Result.chain/2","doc":"Chains a result with a function that returns a new result.\n\nIf the initial result is `{:ok, value}`, the provided function is called with the value,\nand its result is returned. If the initial result is `{:error, error}`, the error is returned\nwithout calling the function.\n\nGreat for building lazy pipelines of fallible computations.","ref":"Lace.Result.html#chain/2"},{"type":"function","title":"Examples - Lace.Result.chain/2","doc":"iex> chain({:ok, 1}, fn x -> {:ok, x + 1} end)\n    {:ok, 2}\n\n    iex> chain({:error, \"missing number\"}, fn x -> {:ok, x + 1} end)\n    {:error, \"missing number\"}\n\n    iex> {:ok, 1}\n    ...> |> chain(fn x -> {:ok, x + 1} end)\n    ...> |> chain(fn x -> {:ok, x * 2} end)\n    ...> |> chain(fn x -> {:error, :nan} end)\n    ...> |> chain(fn x -> {:ok, x * 10} end)\n    {:error, :nan}","ref":"Lace.Result.html#chain/2-examples"},{"type":"function","title":"Lace.Result.collapse/1","doc":"Collapses a list of results into a single result.\n\nStops at the first error encountered.","ref":"Lace.Result.html#collapse/1"},{"type":"function","title":"Examples - Lace.Result.collapse/1","doc":"iex> Lace.Result.collapse([{:ok, 1}, {:ok, 2}, {:ok, 3}])\n    {:ok, [1, 2, 3]}\n\n    iex> Lace.Result.collapse([{:ok, 1}, {:error, 2}, {:ok, 3}, {:error, 4}])\n    {:error, 2}","ref":"Lace.Result.html#collapse/1-examples"},{"type":"function","title":"Lace.Result.collect/1","doc":"Collect a list of results into a tuple of ok values and errors.","ref":"Lace.Result.html#collect/1"},{"type":"function","title":"Examples - Lace.Result.collect/1","doc":"iex> Lace.Result.collect([{:ok, 1}, {:error, 2}, {:ok, 3}])\n    {[1, 3], [2]}\n\n    iex> Lace.Result.collect([{:ok, \"green\"}])\n    {[\"green\"], []}","ref":"Lace.Result.html#collect/1-examples"},{"type":"function","title":"Lace.Result.error/1","doc":"Turns a value into an error `t:result()` by wrapping it in an `{:error, ...}` tuple.\n\nNote that this always returns an :error tuple, it does not try to infer the most\nappropriate result type. Use `from/1` for that.","ref":"Lace.Result.html#error/1"},{"type":"function","title":"Examples - Lace.Result.error/1","doc":"iex> :http_error |> error()\n    {:error, :http_error}\n\n    iex> {:ok, 12} |> error()\n    {:error, {:ok, 12}}","ref":"Lace.Result.html#error/1-examples"},{"type":"function","title":"Lace.Result.from/1","doc":"Tries its best to transform any value into a well-formed `t:result` tuple,\nof the kind handled by the functions in this module.\nIt does so by based on the value's _resultishness_: _errorish_ values are converted\nto `t:error/0`s, while any other value is converted to `t:ok/0`.\n\nUseful to transform _resultish_ values into proper result values,\nwhich can then be used in chainable and linear `Lace.Result` pipelines.","ref":"Lace.Result.html#from/1"},{"type":"function","title":"Examples: - Lace.Result.from/1","doc":"iex> from(12)\n    {:ok, 12}\n\n    iex> from(:ok)\n    {:ok, nil}\n\n    iex> from(:error)\n    {:error, nil}\n\n    iex> from({:ok, 12})\n    {:ok, 12}\n\n    iex> from({:error, :reason, %{trace_id: 100}})\n    {:error, {:reason, %{trace_id: 100}}}\n\n    iex> from(RuntimeError.exception(\"oh-oh!\"))\n    {:error, %RuntimeError{message: \"oh-oh!\"}}","ref":"Lace.Result.html#from/1-examples"},{"type":"macro","title":"Lace.Result.is_errorish/1","doc":"Returns `true` if `val` is _errorish_ (including a well-formed error value).\nSuch values would be converted into an `{:error, _}` tuple by the `from/1` function.\nCan be used in guard tests.","ref":"Lace.Result.html#is_errorish/1"},{"type":"macro","title":"Lace.Result.is_okish/1","doc":"Returns `true` if `val` is _okish_ (including a well-formed ok value).\nSuch values would be converted into an `{:ok, _}` tuple by the `from/1` function.\nCan be used in guard tests.","ref":"Lace.Result.html#is_okish/1"},{"type":"macro","title":"Lace.Result.is_valish/1","doc":"Returns `true` if `val` is neither _okish_ nor _errorish_.\nSuch values would be converted into an `{:ok, _}` tuple by the `from/` function.\nCan be used in guard tests.","ref":"Lace.Result.html#is_valish/1"},{"type":"function","title":"Lace.Result.ok/1","doc":"Turns a value into an ok `t:result()` by wrapping it in an `{:ok, ...}` tuple.\n\nNote that this always returns an :ok tuple, it does not try to infer the most\nappropriate result type. Use `from/1` for that.","ref":"Lace.Result.html#ok/1"},{"type":"function","title":"Examples - Lace.Result.ok/1","doc":"iex> 2 |> ok()\n    {:ok, 2}\n\n    iex> {:error, :http_error} |> ok()\n    {:ok, {:error, :http_error}}","ref":"Lace.Result.html#ok/1-examples"},{"type":"type","title":"Lace.Result.error/1","doc":"","ref":"Lace.Result.html#t:error/1"},{"type":"type","title":"Lace.Result.errorish/0","doc":"","ref":"Lace.Result.html#t:errorish/0"},{"type":"type","title":"Lace.Result.ok/1","doc":"","ref":"Lace.Result.html#t:ok/1"},{"type":"type","title":"Lace.Result.okish/0","doc":"","ref":"Lace.Result.html#t:okish/0"},{"type":"type","title":"Lace.Result.result/2","doc":"","ref":"Lace.Result.html#t:result/2"},{"type":"extras","title":"License","doc":"MIT License\n\nCopyright (c) 2024 Leonardo Donelli\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}